/*  *********************************************************************** */
/*  *********************************************************************** */

/*  FILE monitor.h - *SAT 1.0 */
/*  Performance monitoring */ 

/*  *********************************************************************** */
/*  *********************************************************************** */

#include <stdio.h>

#include "util.h"

#ifndef MONITOR
#define MONITOR

#define MAX_CLOCK   5
#define MAX_PARAM   23

/* Timings (MAX CLOCK) */
#define PARSE       0 /* Time for parsing the input formula */
#define PREPROCESS  1 /* Time for preprocessing the input formula */
#define TRANSLATE   2 /* Time for translating */
#define BUILD       3 /* Time to set up the data structure and initialize it */ 
#define DSAT        4 /* Time to run the decision procedure */

/* Parameters (MAX_PARAM) */

/* General */
#define REC_CALLS     0  /* Number of consistency checks (recursive calls) */ 

/* Davis Putnam */
#define UNIT_NUM      1  /* Number of unit propagations */
#define SPLIT_NUM     2  /* Number of splits */         

/* Caching */
#define CACHE_ACCESS  11 /* Number of cache accesses */
#define CACHE_SUCCESS 12 /* Number of cache successes */

/* Memory usage */
#define MAX_SATO_MEMORY     21 /* Maximum memory allocated by SATO */
#define TOTAL_CACHE_MEMORY  22 /* Total memory used by the cache */

typedef int clock_table_t[MAX_CLOCK]; /* Must accomodate all the timings */
typedef int param_table_t[MAX_PARAM]; /* Must accomodate all the parameters */

typedef struct monitor {
  clock_table_t timings;
  param_table_t params;
} monitor_t;

/* *********************************************************************** */
/* Constructors & Deconstructors */
/* *********************************************************************** */
monitor_t *Init_monitor();
void Clear_monitor(monitor_t *monitor_ptr);

/* *********************************************************************** */
/* CPU times */
/* *********************************************************************** */  
#define Start_clock(monitor_ptr, field) \
monitor_ptr->timings[field] = util_cpu_time()

#define Stop_clock(monitor_ptr, field) \
monitor_ptr->timings[field] = util_cpu_time() - monitor_ptr->timings[field]

#define Lap_time(monitor_ptr, field) \
(util_cpu_time() - monitor_ptr->timings[field])

#define Print_clock(monitor_ptr, field) \
util_print_time(monitor_ptr->timings[field])

/* *********************************************************************** */
/* Parameters */
/* *********************************************************************** */
#define Inc_param(monitor_ptr, field) (monitor_ptr->params[field])++
#define Inc_param_by(monitor_ptr, field, inc) (monitor_ptr->params[field])+=inc

#define Get_param(monitor_ptr, field) (monitor_ptr->params[field])

#endif


